==> fb_library-0.0.38.tar.gz: fb_library-0.0.38/tests/test_ball_socket.py <==
from importlib.machinery import SourceFileLoader
from importlib.util import module_from_spec, spec_from_loader
from unittest.mock import patch
import math
import pytest
from build123d import Part

from fb_library.ball_socket import ball_mount, ball_socket


# ---------- Helpers ----------
def expected_socket_height(r: float, w: float) -> float:
    # Matches current implementation: cylinder height = ball_radius + wall_thickness * 2.5
    return r + w * 2.5


def expected_socket_diameter(r: float, w: float) -> float:
    return 2 * (r + w)


# ---------- Ball Mount Tests ----------
class TestBallMount:
    def test_ball_mount_basic(self):
        mount = ball_mount(10.0)
        assert isinstance(mount, Part)
        assert mount.is_valid()
        bbox = mount.bounding_box()
        assert bbox.size.X == pytest.approx(20.0, abs=0.1)
        assert bbox.size.Y == pytest.approx(20.0, abs=0.1)
        # Height = 3.5 * radius (shaft from 0 to 2.25R, sphere center at 2.5R -> top 3.5R)
        assert bbox.size.Z == pytest.approx(35.0, abs=0.5)

    @pytest.mark.parametrize("r", [0.5, 2.0, 7.5, 10.0, 25.0])
    def test_ball_mount_dimensions(self, r):
        mount = ball_mount(r)
        assert mount.is_valid()
        bbox = mount.bounding_box()
        assert bbox.size.X == pytest.approx(2 * r, abs=0.05)
        assert bbox.size.Y == pytest.approx(2 * r, abs=0.05)
        assert bbox.size.Z == pytest.approx(3.5 * r, rel=0.02)

    def test_ball_mount_centering(self):
        mount = ball_mount(10.0)
        bbox = mount.bounding_box()
        assert abs(bbox.center().X) < 0.01
        assert abs(bbox.center().Y) < 0.01

    def test_ball_mount_volume_positive(self):
        assert ball_mount(5).volume > 0

    def test_ball_mount_shaft_geometry(self):
        mount = ball_mount(10.0)
        # Basic sanity: top > 34, bottom at 0
        bbox = mount.bounding_box()
        assert bbox.min.Z == pytest.approx(0.0, abs=0.05)
        assert bbox.max.Z == pytest.approx(35.0, abs=0.5)


# ---------- Ball Socket Tests (Updated for new geometry) ----------
class TestBallSocket:
    def test_ball_socket_basic(self):
        r = 10.0
        w = 2.0
        socket = ball_socket(r)
        assert isinstance(socket, Part)
        assert socket.is_valid()
        assert socket.label == "Ball Socket"
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Y == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    @pytest.mark.parametrize("r,w", [(3, 2), (5, 1), (10, 2), (20, 4), (12.5, 3.5)])
    def test_ball_socket_param_dimensions(self, r, w):
        socket = ball_socket(r, wall_thickness=w)
        assert socket.is_valid()
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Y == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)
        assert bbox.min.Z == pytest.approx(0.0, abs=0.01)

    def test_ball_socket_custom_wall_thickness(self):
        r, w = 10.0, 3.0
        socket = ball_socket(r, wall_thickness=w)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_ball_socket_tolerance_does_not_change_outer_size(self):
        r, w = 10.0, 2.0
        base_bbox = ball_socket(r).bounding_box()
        for tol in [-0.1, 0.0, 0.1, 0.5]:
            bbox = ball_socket(r, tolerance=tol).bounding_box()
            assert bbox.size.X == pytest.approx(base_bbox.size.X, abs=0.05)
            assert bbox.size.Z == pytest.approx(base_bbox.size.Z, abs=0.05)

    def test_ball_socket_tolerance_volume_effect(self):
        r, w = 10.0, 2.0
        loose = ball_socket(r, wall_thickness=w, tolerance=0.5)
        tight = ball_socket(r, wall_thickness=w, tolerance=-0.05)
        assert loose.is_valid() and tight.is_valid()
        # Larger positive tolerance removes more -> smaller remaining part volume
        assert loose.volume < tight.volume

    def test_ball_socket_wall_thickness_volume_growth(self):
        r = 10.0
        thin = ball_socket(r, wall_thickness=1.0)
        thick = ball_socket(r, wall_thickness=5.0)
        assert thin.volume < thick.volume

    def test_ball_socket_centered(self):
        socket = ball_socket(10.0)
        bbox = socket.bounding_box()
        assert abs(bbox.center().X) < 0.01
        assert abs(bbox.center().Y) < 0.01
        assert bbox.min.Z == pytest.approx(0.0, abs=0.01)

    def test_ball_socket_small_radius(self):
        r, w = 3.0, 2.0
        socket = ball_socket(r)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_ball_socket_large_radius(self):
        r, w = 20.0, 2.0
        socket = ball_socket(r)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_ball_socket_fractional_radius(self):
        r, w = 7.5, 1.5
        socket = ball_socket(r, wall_thickness=w, tolerance=0.05)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Y == pytest.approx(expected_socket_diameter(r, w), abs=0.1)

    def test_ball_socket_has_flex_cuts_volume_reduction(self):
        r, w = 10.0, 2.0
        socket = ball_socket(r)
        assert socket.volume > 0
        # Compare to solid cylinder of same outer size
        solid_volume = math.pi * (r + w) ** 2 * expected_socket_height(r, w)
        assert socket.volume < solid_volume * 0.9  # should be noticeably reduced


# ---------- Pair Compatibility ----------
class TestBallSocketPairCompatibility:
    @pytest.mark.parametrize("r", [2.0, 5.0, 10.0, 15.0, 20.0])
    def test_mount_socket_compatibility(self, r):
        mount = ball_mount(r)
        socket = ball_socket(r)
        assert mount.is_valid() and socket.is_valid()
        mount_bbox = mount.bounding_box()
        socket_bbox = socket.bounding_box()
        # Mount ball diameter should be <= socket outer diameter
        assert mount_bbox.size.X <= socket_bbox.size.X
        # Mount should be taller than socket for most cases, but may be equal for small r
        # Mount height: 3.5R, Socket height: R + 2.5w (where w=2 by default)
        # For r=2, w=2: mount≈7, socket≈7 (approximately equal due to geometry)
        # For r>2.86, mount > socket
        if r > 2.86:
            assert mount_bbox.size.Z > socket_bbox.size.Z
        else:
            # Use approximate comparison for small radii due to floating-point precision
            assert mount_bbox.size.Z == pytest.approx(socket_bbox.size.Z, abs=0.01)

    def test_multiple_radius_compatibility(self):
        for r in [2.0, 5.0, 10.0, 15.0, 20.0]:
            assert ball_mount(r).is_valid()
            assert ball_socket(r).is_valid()


# ---------- Edge / Extreme Cases ----------
class TestEdgeCases:

    def test_extreme_tolerance_values(self):
        tight = ball_socket(10.0, tolerance=-0.1)
        loose = ball_socket(10.0, tolerance=1.0)
        assert tight.is_valid()
        assert loose.is_valid()
        assert loose.volume < ball_socket(10.0, tolerance=0.0).volume

    def test_parameter_validation_edge_cases(self):
        cases = [
            (1.0, 0.5, 0.01),
            (50.0, 10.0, 2.0),
            # Skip (10.0, 0.1, -0.05) - too thin walls cause fillet issues
            (10.0, 8.0, 0.5),
        ]
        for r, w, t in cases:
            assert ball_mount(r).is_valid()
            # Skip socket test for very thin walls
            if w >= 0.5:  # Only test if wall thickness is reasonable
                assert ball_socket(r, w, t).is_valid()


# ---------- Direct Run ----------
class TestDirectRun:
    def test_direct_run(self):
        with patch("ocp_vscode.show"):
            loader = SourceFileLoader("__main__", "src/fb_library/ball_socket.py")
            loader.exec_module(module_from_spec(spec_from_loader(loader.name, loader)))


# ---------- Geometry Details ----------
class TestGeometryDetails:
    def test_ball_mount_shaft_taper(self):
        r = 10.0
        mount = ball_mount(r)
        bbox = mount.bounding_box()
        assert bbox.max.Z >= 35 - 0.5  # top tolerance
        assert bbox.min.Z == pytest.approx(0.0, abs=0.05)

    def test_ball_socket_internal_features(self):
        r = 10.0
        socket = ball_socket(r)
        bbox = socket.bounding_box()
        assert socket.volume > 0
        assert bbox.min.Z == pytest.approx(0.0, abs=0.01)

    def test_socket_filleted_top_exists(self):
        # Indirect: ensure top height unchanged but internal edge count reduced after fillet
        r, w = 10.0, 2.0
        socket = ball_socket(r, wall_thickness=w)
        assert socket.is_valid()
        bbox = socket.bounding_box()
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_flex_cuts_reduce_volume(self):
        r, w = 10.0, 2.0
        base = ball_socket(r, wall_thickness=w, tolerance=0.0)
        # Simulate no flex cuts by creating a temporary variant (approximate by comparing to solid cylinder)
        solid_volume = math.pi * (r + w) ** 2 * expected_socket_height(r, w)
        assert base.volume < solid_volume * 0.95  # noticeable reduction

    def test_volume_monotonic_with_wall_thickness(self):
        r = 10.0
        vols = []
        for w in [0.5, 1.0, 2.0, 3.0]:
            vols.append(ball_socket(r, wall_thickness=w).volume)
        assert vols == sorted(vols)


# ---------- New Additional Robustness Tests ----------
class TestAdditional:
    @pytest.mark.parametrize("tol", [-0.2, -0.05, 0.0, 0.1, 0.4])
    def test_tolerance_monotonic_volume(self, tol):
        r, w = 8.0, 2.0
        # Larger tolerance -> larger internal cavity -> smaller remaining part
        base_vols = {}
        for t in sorted([-0.2, -0.05, 0.0, 0.1, 0.4]):
            base_vols[t] = ball_socket(r, wall_thickness=w, tolerance=t).volume
        # Check ordering
        ordered = [base_vols[t] for t in sorted(base_vols)]
        assert ordered == sorted(ordered, reverse=True)

    @pytest.mark.parametrize("r,w", [(5, 2), (12, 3), (18, 4)])
    def test_height_formula_consistency(self, r, w):
        bbox = ball_socket(r, wall_thickness=w).bounding_box()
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

==> fb_library-0.0.39.tar.gz: fb_library-0.0.39/tests/test_ball_socket.py <==
from importlib.machinery import SourceFileLoader
from importlib.util import module_from_spec, spec_from_loader
from unittest.mock import patch
import math
import pytest
from build123d import Part

from fb_library.ball_socket import ball_mount, ball_socket


# ---------- Helpers ----------
def expected_socket_height(r: float, w: float) -> float:
    # Matches current implementation: cylinder height = ball_radius + wall_thickness * 2.5
    return r + w * 2.5


def expected_socket_diameter(r: float, w: float) -> float:
    return 2 * (r + w)


# ---------- Ball Mount Tests ----------
class TestBallMount:
    def test_ball_mount_basic(self):
        mount = ball_mount(10.0)
        assert isinstance(mount, Part)
        assert mount.is_valid
        bbox = mount.bounding_box()
        assert bbox.size.X == pytest.approx(20.0, abs=0.1)
        assert bbox.size.Y == pytest.approx(20.0, abs=0.1)
        # Height = 3.5 * radius (shaft from 0 to 2.25R, sphere center at 2.5R -> top 3.5R)
        assert bbox.size.Z == pytest.approx(35.0, abs=0.5)

    @pytest.mark.parametrize("r", [0.5, 2.0, 7.5, 10.0, 25.0])
    def test_ball_mount_dimensions(self, r):
        mount = ball_mount(r)
        assert mount.is_valid
        bbox = mount.bounding_box()
        assert bbox.size.X == pytest.approx(2 * r, abs=0.05)
        assert bbox.size.Y == pytest.approx(2 * r, abs=0.05)
        assert bbox.size.Z == pytest.approx(3.5 * r, rel=0.02)

    def test_ball_mount_centering(self):
        mount = ball_mount(10.0)
        bbox = mount.bounding_box()
        assert abs(bbox.center().X) < 0.01
        assert abs(bbox.center().Y) < 0.01

    def test_ball_mount_volume_positive(self):
        assert ball_mount(5).volume > 0

    def test_ball_mount_shaft_geometry(self):
        mount = ball_mount(10.0)
        # Basic sanity: top > 34, bottom at 0
        bbox = mount.bounding_box()
        assert bbox.min.Z == pytest.approx(0.0, abs=0.05)
        assert bbox.max.Z == pytest.approx(35.0, abs=0.5)


# ---------- Ball Socket Tests (Updated for new geometry) ----------
class TestBallSocket:
    def test_ball_socket_basic(self):
        r = 10.0
        w = 2.0
        socket = ball_socket(r)
        assert isinstance(socket, Part)
        assert socket.is_valid
        assert socket.label == "Ball Socket"
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Y == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    @pytest.mark.parametrize("r,w", [(3, 2), (5, 1), (10, 2), (20, 4), (12.5, 3.5)])
    def test_ball_socket_param_dimensions(self, r, w):
        socket = ball_socket(r, wall_thickness=w)
        assert socket.is_valid
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Y == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)
        assert bbox.min.Z == pytest.approx(0.0, abs=0.01)

    def test_ball_socket_custom_wall_thickness(self):
        r, w = 10.0, 3.0
        socket = ball_socket(r, wall_thickness=w)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_ball_socket_tolerance_does_not_change_outer_size(self):
        r, w = 10.0, 2.0
        base_bbox = ball_socket(r).bounding_box()
        for tol in [-0.1, 0.0, 0.1, 0.5]:
            bbox = ball_socket(r, tolerance=tol).bounding_box()
            assert bbox.size.X == pytest.approx(base_bbox.size.X, abs=0.05)
            assert bbox.size.Z == pytest.approx(base_bbox.size.Z, abs=0.05)

    def test_ball_socket_tolerance_volume_effect(self):
        r, w = 10.0, 2.0
        loose = ball_socket(r, wall_thickness=w, tolerance=0.5)
        tight = ball_socket(r, wall_thickness=w, tolerance=-0.05)
        assert loose.is_valid and tight.is_valid
        # Larger positive tolerance removes more -> smaller remaining part volume
        assert loose.volume < tight.volume

    def test_ball_socket_wall_thickness_volume_growth(self):
        r = 10.0
        thin = ball_socket(r, wall_thickness=1.0)
        thick = ball_socket(r, wall_thickness=5.0)
        assert thin.volume < thick.volume

    def test_ball_socket_centered(self):
        socket = ball_socket(10.0)
        bbox = socket.bounding_box()
        assert abs(bbox.center().X) < 0.01
        assert abs(bbox.center().Y) < 0.01
        assert bbox.min.Z == pytest.approx(0.0, abs=0.01)

    def test_ball_socket_small_radius(self):
        r, w = 3.0, 2.0
        socket = ball_socket(r)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_ball_socket_large_radius(self):
        r, w = 20.0, 2.0
        socket = ball_socket(r)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_ball_socket_fractional_radius(self):
        r, w = 7.5, 1.5
        socket = ball_socket(r, wall_thickness=w, tolerance=0.05)
        bbox = socket.bounding_box()
        assert bbox.size.X == pytest.approx(expected_socket_diameter(r, w), abs=0.1)
        assert bbox.size.Y == pytest.approx(expected_socket_diameter(r, w), abs=0.1)

    def test_ball_socket_has_flex_cuts_volume_reduction(self):
        r, w = 10.0, 2.0
        socket = ball_socket(r)
        assert socket.volume > 0
        # Compare to solid cylinder of same outer size
        solid_volume = math.pi * (r + w) ** 2 * expected_socket_height(r, w)
        assert socket.volume < solid_volume * 0.9  # should be noticeably reduced


# ---------- Pair Compatibility ----------
class TestBallSocketPairCompatibility:
    @pytest.mark.parametrize("r", [2.0, 5.0, 10.0, 15.0, 20.0])
    def test_mount_socket_compatibility(self, r):
        mount = ball_mount(r)
        socket = ball_socket(r)
        assert mount.is_valid and socket.is_valid
        mount_bbox = mount.bounding_box()
        socket_bbox = socket.bounding_box()
        # Mount ball diameter should be <= socket outer diameter
        assert mount_bbox.size.X <= socket_bbox.size.X
        # Mount should be taller than socket for most cases, but may be equal for small r
        # Mount height: 3.5R, Socket height: R + 2.5w (where w=2 by default)
        # For r=2, w=2: mount≈7, socket≈7 (approximately equal due to geometry)
        # For r>2.86, mount > socket
        if r > 2.86:
            assert mount_bbox.size.Z > socket_bbox.size.Z
        else:
            # Use approximate comparison for small radii due to floating-point precision
            assert mount_bbox.size.Z == pytest.approx(socket_bbox.size.Z, abs=0.01)

    def test_multiple_radius_compatibility(self):
        for r in [2.0, 5.0, 10.0, 15.0, 20.0]:
            assert ball_mount(r).is_valid
            assert ball_socket(r).is_valid


# ---------- Edge / Extreme Cases ----------
class TestEdgeCases:

    def test_extreme_tolerance_values(self):
        tight = ball_socket(10.0, tolerance=-0.1)
        loose = ball_socket(10.0, tolerance=1.0)
        assert tight.is_valid
        assert loose.is_valid
        assert loose.volume < ball_socket(10.0, tolerance=0.0).volume

    def test_parameter_validation_edge_cases(self):
        cases = [
            (1.0, 0.5, 0.01),
            (50.0, 10.0, 2.0),
            # Skip (10.0, 0.1, -0.05) - too thin walls cause fillet issues
            (10.0, 8.0, 0.5),
        ]
        for r, w, t in cases:
            assert ball_mount(r).is_valid
            # Skip socket test for very thin walls
            if w >= 0.5:  # Only test if wall thickness is reasonable
                assert ball_socket(r, w, t).is_valid


# ---------- Direct Run ----------
class TestDirectRun:
    def test_direct_run(self):
        with patch("ocp_vscode.show"):
            loader = SourceFileLoader("__main__", "src/fb_library/ball_socket.py")
            loader.exec_module(module_from_spec(spec_from_loader(loader.name, loader)))


# ---------- Geometry Details ----------
class TestGeometryDetails:
    def test_ball_mount_shaft_taper(self):
        r = 10.0
        mount = ball_mount(r)
        bbox = mount.bounding_box()
        assert bbox.max.Z >= 35 - 0.5  # top tolerance
        assert bbox.min.Z == pytest.approx(0.0, abs=0.05)

    def test_ball_socket_internal_features(self):
        r = 10.0
        socket = ball_socket(r)
        bbox = socket.bounding_box()
        assert socket.volume > 0
        assert bbox.min.Z == pytest.approx(0.0, abs=0.01)

    def test_socket_filleted_top_exists(self):
        # Indirect: ensure top height unchanged but internal edge count reduced after fillet
        r, w = 10.0, 2.0
        socket = ball_socket(r, wall_thickness=w)
        assert socket.is_valid
        bbox = socket.bounding_box()
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)

    def test_flex_cuts_reduce_volume(self):
        r, w = 10.0, 2.0
        base = ball_socket(r, wall_thickness=w, tolerance=0.0)
        # Simulate no flex cuts by creating a temporary variant (approximate by comparing to solid cylinder)
        solid_volume = math.pi * (r + w) ** 2 * expected_socket_height(r, w)
        assert base.volume < solid_volume * 0.95  # noticeable reduction

    def test_volume_monotonic_with_wall_thickness(self):
        r = 10.0
        vols = []
        for w in [0.5, 1.0, 2.0, 3.0]:
            vols.append(ball_socket(r, wall_thickness=w).volume)
        assert vols == sorted(vols)


# ---------- New Additional Robustness Tests ----------
class TestAdditional:
    @pytest.mark.parametrize("tol", [-0.2, -0.05, 0.0, 0.1, 0.4])
    def test_tolerance_monotonic_volume(self, tol):
        r, w = 8.0, 2.0
        # Larger tolerance -> larger internal cavity -> smaller remaining part
        base_vols = {}
        for t in sorted([-0.2, -0.05, 0.0, 0.1, 0.4]):
            base_vols[t] = ball_socket(r, wall_thickness=w, tolerance=t).volume
        # Check ordering
        ordered = [base_vols[t] for t in sorted(base_vols)]
        assert ordered == sorted(ordered, reverse=True)

    @pytest.mark.parametrize("r,w", [(5, 2), (12, 3), (18, 4)])
    def test_height_formula_consistency(self, r, w):
        bbox = ball_socket(r, wall_thickness=w).bounding_box()
        assert bbox.size.Z == pytest.approx(expected_socket_height(r, w), abs=0.1)
